package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.os.Bundle;
import androidx.annotation.Nullable;
import android.view.Gravity;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;

/**
 * BaseActivity.java
 * Base class for dialer activities
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-04-19
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public abstract class BaseActivity extends AppCompatActivity {

  @Nullable
  @BindView(R.id.toolbar)
  Toolbar toolbar;

  private Unbinder unbinder;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(getLayoutResource());
    unbinder = ButterKnife.bind(this);

    if (toolbar != null) {
      setSupportActionBar(toolbar);
      ActionBar supportActionBar = getSupportActionBar();
      if (supportActionBar != null) {
        supportActionBar.setDisplayHomeAsUpEnabled(false);
        String title = null;
        if (savedInstanceState != null) {
          title = savedInstanceState.getString("title");
        }
        supportActionBar.setTitle(title);
      }
    }
  }

  public void customizeToolbar() {
    for (int i = 0; i < toolbar.getChildCount(); i++) {
      View view = toolbar.getChildAt(i);
      if (view instanceof TextView) {
        Toolbar.LayoutParams params = new Toolbar.LayoutParams(Toolbar.LayoutParams.WRAP_CONTENT,
            Toolbar.LayoutParams.MATCH_PARENT);
        params.gravity = Gravity.CENTER_HORIZONTAL;
        view.setLayoutParams(params);
      }
    }
  }

  @Override
  protected void onDestroy() {
    super.onDestroy();
    unbinder.unbind();
  }

  @Override
  protected void onSaveInstanceState(Bundle outState) {
    super.onSaveInstanceState(outState);
    ActionBar supportActionBar = getSupportActionBar();
    if (supportActionBar != null) {
      CharSequence title = supportActionBar.getTitle();
      if (title != null) {
        outState.putString("title", title.toString());
      }
    }
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
      case android.R.id.home:
        onBackPressed();
        return true;
      default:
        return super.onOptionsItemSelected(item);
    }
  }

  public void enableBackButton() {
    if (getSupportActionBar() != null) {
      getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }
  }

  public void setTitleRes(int title) {
    if (getSupportActionBar() != null) {
      setTitle(getString(title));
    }
  }

  public void setTitle(String title) {
    if (getSupportActionBar() != null) {
      getSupportActionBar().setTitle(title);
    }
  }

  protected abstract int getLayoutResource();

  public void addFragment(Fragment fragment, int containerId, String tag) {
    FragmentManager fragmentManager = getSupportFragmentManager();
    fragmentManager.beginTransaction()
        .setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out,
            android.R.anim.fade_in, android.R.anim.fade_out)
        .add(containerId, fragment, tag)
        .addToBackStack(tag)
        .commit();
  }

}
