package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import static net.greenfieldtech.cloudonixsdk.appinterface.CloudonixCallType.REGISTRATION_FREE_CALL;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.text.TextUtils;
import android.util.Log;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import java.util.Date;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.api.models.SDKConstants.AudioRoute;
import net.greenfieldtech.cloudonixsdk.appinterface.CloudonixSDKClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.api.AppRestClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.database.DatabaseHelper;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RecentCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.Ringer;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;

/**
 * IncomingCallActivity.java
 * Incoming call view
 * 
 * Authors:
 *   - Aleksei Dzis, 2016-08-03
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class IncomingCallActivity extends AppCompatActivity {

  private static final int MINUTE = 60000;
  private static final String TAG = "IncomingCallActivity";

  @BindView(R.id.tv_current_call_number)
  TextView tvNumber;
  @BindView(R.id.tv_contact_name)
  TextView tvContactName;

  private String callId;
  private String callNumber;
  private String sessionId;
  private Ringer mRinger;
  private WakeLock mWakeLock;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    requestWindowFeature(Window.FEATURE_NO_TITLE);

    mRinger = new Ringer(this);

    getWindow().addFlags(LayoutParams.FLAG_SHOW_WHEN_LOCKED
        | LayoutParams.FLAG_DISMISS_KEYGUARD
        | LayoutParams.FLAG_KEEP_SCREEN_ON
        | LayoutParams.FLAG_TURN_SCREEN_ON
        | LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON);

    setContentView(R.layout.activity_incoming_call);
    ButterKnife.bind(this);

    Intent intent = getIntent();
    callNumber = intent.getStringExtra(CloudonixSDKClient.EXTRA_CALL_CONTACT);

    if (intent.getStringExtra(CloudonixSDKClient.EXTRA_CALL_TYPE) != null) {
      if (intent.getStringExtra(CloudonixSDKClient.EXTRA_CALL_TYPE).equals(REGISTRATION_FREE_CALL)) {
        sessionId = intent.getStringExtra(CloudonixSDKClient.EXTRA_CALL_SESSION_ID);
      }
    } else {
      callId = intent.getStringExtra(CloudonixSDKClient.EXTRA_CALL_KEY);
      if (callId == null) {
        finish();
      }
    }

    tvNumber.setText(callNumber);
    tvContactName.setText("Name - " + callNumber);

    if (VoIPClient.getInstance() == null) {
      VoIPClient.initWithContext(getApplicationContext());
    }
    super.onCreate(savedInstanceState);
  }

  private void observeRingingNotification() {
    String msisdn = AppPrefs.getInstance().getString(AppPrefsKeys.USER_NAME);
    String sipServer = AppPrefs.getInstance().getString(AppPrefsKeys.SERVER);
    String domain = AppPrefs.getInstance().getString(AppPrefsKeys.DOMAIN);
    AppRestClient.getRingingNotification(msisdn, sipServer, domain, sessionId, this::finish);
  }

  @OnClick(R.id.btn_answer)
  void answer() {
    DatabaseHelper databaseHelper = DatabaseHelper.getInstance(this);
    RecentCall call = new RecentCall("Name - " + callNumber, callNumber, new Date());
    call.setIncoming(true);
    call.setDeclined(false);
    databaseHelper.addCall(call);

    Intent startCallActivity = new Intent(this, CallActivity.class);
    startCallActivity.putExtra("call_number", callNumber);
    if (!TextUtils.isEmpty(sessionId)) {
      VoIPClient.getInstance().dialRegistrationFree(callNumber, sessionId);
    } else {
      startCallActivity.putExtra("call_id", callId);
      VoIPClient.getInstance().answer(callId);
    }

    startActivity(startCallActivity);
    finishWithResult(CallActivity.CALL_HANGUP);
  }

  @OnClick(R.id.btn_decline)
  void decline() {
    DatabaseHelper databaseHelper = DatabaseHelper.getInstance(this);
    RecentCall call = new RecentCall("Name - " + callNumber, callNumber, new Date());
    call.setIncoming(true);
    call.setDeclined(true);
    databaseHelper.addCall(call);

    if (!TextUtils.isEmpty(sessionId)) {
      String msisdn = AppPrefs.getInstance().getString(AppPrefsKeys.USER_NAME);
      String pass = AppPrefs.getInstance().getString(AppPrefsKeys.PASSWORD);
      String sipServer = AppPrefs.getInstance().getString(AppPrefsKeys.SERVER);
      String domain = AppPrefs.getInstance().getString(AppPrefsKeys.DOMAIN);
      AppRestClient.deleteIncomingSession(msisdn, pass, sipServer, domain, sessionId);
    } else {
      VoIPClient.getInstance().reject(callId);
    }

    finishWithResult(CallActivity.CALL_HANGUP);
  }

  @Override
  protected void onResume() {
    super.onResume();
    VoIPClient.getInstance().addCallbacksListener(callback);

    PowerManager powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
    mWakeLock = powerManager
        .newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ACQUIRE_CAUSES_WAKEUP,
            "cloudonixdialer:RINGERWAKELOCK");
    mWakeLock.acquire(MINUTE);
    mRinger.startRinging();
    if (VoIPClient.getInstance().getCallState(callId) == null) {
      if (isTaskRoot()) {
        startActivity(new Intent(this, MainActivity.class));
      }
      finishWithResult(CallActivity.CALL_HANGUP);
    }
  }

  @Override
  protected void onPause() {
    VoIPClient.getInstance().removeCallbacksListener(callback);
    if (mWakeLock.isHeld()) {
      mWakeLock.release();
    }
    mRinger.stopRinging();
    super.onPause();
  }

  @Override
  public void onBackPressed() {
    DatabaseHelper databaseHelper = DatabaseHelper.getInstance(IncomingCallActivity.this);
    RecentCall call = new RecentCall("Name - " + callNumber, callNumber, new Date());
    call.setIncoming(true);
    call.setDeclined(true);
    databaseHelper.addCall(call);

    VoIPClient.getInstance().reject(callId);
    super.onBackPressed();
  }

  private void finishWithResult(int result) {
    Bundle conData = new Bundle();
    Intent intent = new Intent();
    intent.putExtras(conData);
    setResult(result, intent);
    finish();
  }

  private IVoIPObserver callback = new IVoIPObserver() {

    @Override
    public void onSipStarted() {

    }

    @Override
    public void onSipStartFailed() {

    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {

    }

    @Override
    public void onCallState(String key, CallState callState, String contactNumber) {
      Log.d(TAG, "onCallState: [" + callId + "] " + callState);
      switch (callState) {
        case CALL_STATE_DISCONNECTED:
        case CALL_STATE_DISCONNECTEDDUETOBUSY:
        case CALL_STATE_DISCONNECTEDDUETOTIMEOUT:
        case CALL_STATE_DISCONNECTEDMEDIACHANGED:
        case CALL_STATE_DISCONNECTEDDUETONOMEDIA:
        case CALL_STATE_DISCONNECTEDDUETONETWORKCHANGE:

          DatabaseHelper databaseHelper = DatabaseHelper.getInstance(IncomingCallActivity.this);
          RecentCall call = new RecentCall("Name - " + contactNumber, contactNumber, new Date());
          call.setIncoming(true);
          call.setMissed(true);
          call.setViewed(false);
          databaseHelper.addCall(call);

          if (key.equals(callId)) {
            finishWithResult(CallActivity.CALL_HANGUP);
          }
          break;
      }
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {

    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {

    }

    @Override
    public void onNetworkRegained() {

    }

    @Override
    public void onLicense(LicensingState state, String description) {

    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }
  };

}


