package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Window;
import android.view.WindowManager;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.api.models.SDKConstants.AudioRoute;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;

/**
 * LockScreenActivity.java
 * 
 * 
 * Authors:
 *   - Aleksei Dzis, 2016-08-03
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class LockScreenActivity extends Activity implements SensorEventListener {

  private final static float PROXIMITY_DISTANCE_FAR_IN_CM = (float) 5;
  private final static int PROXIMITY_FALSE_POSITIVE_GRACE_MS = 100;
  private static final String TAG = "LockScreenActivity";
  
  private SensorManager mSensorManager;
  private Sensor mProximity;
  private Handler mHandler = new Handler() {
    @Override
    public void handleMessage(Message msg) {
      switch (msg.what) {
        case 0:
          proximityOff();
          break;
      }
    }
  };

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
    mProximity = mSensorManager.getDefaultSensor(Sensor.TYPE_PROXIMITY);
    requestWindowFeature(Window.FEATURE_NO_TITLE);
    getWindow().setFlags(
        WindowManager.LayoutParams.FLAG_FULLSCREEN | WindowManager.LayoutParams.FLAG_BLUR_BEHIND,
        WindowManager.LayoutParams.FLAG_FULLSCREEN);
    getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN
        | WindowManager.LayoutParams.FLAG_BLUR_BEHIND
        | WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
        | WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED
        | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON);
    setContentView(R.layout.activity_lockscreen);
  }

  @Override
  protected void onPause() {
    VoIPClient.getInstance().removeCallbacksListener(callback);
    mSensorManager.unregisterListener(this);
    super.onPause();
  }

  @Override
  protected void onResume() {
    VoIPClient.getInstance().addCallbacksListener(callback);
    mSensorManager.registerListener(this, mProximity, SensorManager.SENSOR_DELAY_NORMAL);
    super.onResume();
  }

  @Override
  public void onBackPressed() {

  }

  private void finishWithResult(int result) {
    Bundle conData = new Bundle();
    Intent intent = new Intent();
    intent.putExtras(conData);
    setResult(result, intent);
    finish();
  }

  @Override
  public void onSensorChanged(SensorEvent event) {
    float distance = event.values[0];

    if (distance < PROXIMITY_DISTANCE_FAR_IN_CM) {
      mHandler.removeMessages(0);
    } else {
      mHandler.sendEmptyMessageDelayed(0, PROXIMITY_FALSE_POSITIVE_GRACE_MS);
    }
  }

  public void proximityOff() {
    Log.i("PROX", "proximity off");
    finishWithResult(CallActivity.CALL_PROXIMITY_OFF);
  }

  @Override
  public void onAccuracyChanged(Sensor sensor, int accuracy) {

  }

  private IVoIPObserver callback = new IVoIPObserver() {

    @Override
    public void onSipStarted() {

    }

    @Override
    public void onSipStartFailed() {

    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {

    }

    @Override
    public void onCallState(String callId, CallState callState, String contactNumber) {
      Log.d(TAG, "onCallState: [" + callId + "] " + callState);
      switch (callState) {
        case CALL_STATE_DISCONNECTED:
        case CALL_STATE_DISCONNECTEDDUETOBUSY:
        case CALL_STATE_DISCONNECTEDDUETOTIMEOUT:
        case CALL_STATE_DISCONNECTEDMEDIACHANGED:
        case CALL_STATE_DISCONNECTEDDUETONOMEDIA:
        case CALL_STATE_DISCONNECTEDDUETONETWORKCHANGE:
          finishWithResult(CallActivity.CALL_HANGUP);
          break;
        default:
          break;
      }
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {

    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {

    }

    @Override
    public void onNetworkRegained() {

    }

    @Override
    public void onLicense(LicensingState state, String description) {

    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }

  };

}
