package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.RelativeLayout;
import android.widget.TabHost;
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.preference.PreferenceManager;
import androidx.viewpager.widget.ViewPager;
import butterknife.BindView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.api.models.SDKConstants.AudioRoute;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.adapters.TabsAdapter;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.events.UpdateSettingsEvent;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.ContactsFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.KeypadFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.RecentCallsFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.StatusFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RegistrationFreeCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.NetManager;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.Utils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.views.TabIndicator;
import net.greenfieldtech.cloudonixsdk.utils.SDKLogger;
import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

/**
 * MainActivity.java
 * Dialer UI
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-04-20
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class MainActivity extends BaseActivity {

  private static final String TAG = "MainActivity";

  @BindView(R.id.root)
  RelativeLayout root;
  @BindView(R.id.pager)
  ViewPager viewPager;
  @BindView(android.R.id.tabhost)
  TabHost tabHost;

  private String mRingingCallId = null;

  private TabsAdapter tabsAdapter;
  private NetworkChangeReceiver networkReceiver;
  private DatabaseReference registrationFreeUser;
  private boolean isBackPressed;

  @Override
  protected int getLayoutResource() {
    return R.layout.activity_main;
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    VoIPClient.initWithContext(getApplicationContext());

    setTitleRes(R.string.main_toolbar_title);

    PreferenceManager.setDefaultValues(this, R.xml.preferences_screen, false);

    if (checkCallingOrSelfPermission(Manifest.permission.READ_CONTACTS)
        != PackageManager.PERMISSION_GRANTED) {
      ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_CONTACTS}, 100);
    } else {
      setupViews();
    }

    if (SdkFacade.findActiveCall(null) != null) {
      String key = SdkFacade.findActiveCall(null);
      String number = VoIPClient.getInstance().getCallUrl(key);

      Intent startCallActivity = new Intent(this, CallActivity.class);
      startCallActivity.putExtra("call_id", key);
      startCallActivity.putExtra("call_number", number);
      startActivity(startCallActivity);
    } else {
      if (VoIPClient.getInstance().isRegistered()
          && VoIPClient.getInstance().getCalls() != null
          && VoIPClient.getInstance().getCalls().length != 0) {
        // TODO check multiple calls
        String callId = VoIPClient.getInstance().getCalls()[0].key;
        String callNumber = VoIPClient.getInstance().getCalls()[0].url;

        Intent startCallActivity = new Intent(this, OutgoingCallActivity.class);
        startCallActivity.putExtra("call_id", callId);
        startCallActivity.putExtra("call_number", callNumber);
        startActivity(startCallActivity);
      }
    }
  }

  @Override
  protected void onDestroy() {
    super.onDestroy();
    try {
      unregisterReceiver(networkReceiver);
    } catch (Exception e) {
//      e.printStackTrace();
    }
    if (!isBackPressed) {
      VoIPClient.getInstance().shutdown();
    }
  }

  private void setupViews() {
    if (tabsAdapter == null) {
      tabHost.setup();
      tabsAdapter = new TabsAdapter(this, tabHost, viewPager);
      initTabs(tabHost);
      viewPager.setCurrentItem(tabsAdapter.getCount() - 1);
      tabsAdapter.setOnTabChangeListener(onTabChangeListener);
    }
  }

  private void initTabs(TabHost mTabHost) {
    TabIndicator status = new TabIndicator(this);
    status.setImage(R.drawable.tab_status_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Status").setIndicator(status), StatusFragment.class, null);

    TabIndicator buyData = new TabIndicator(this);
    buyData.setImage(R.drawable.tab_recent_calls_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("RecentCalls").setIndicator(buyData), RecentCallsFragment.class,
            null);

    TabIndicator save = new TabIndicator(this);
    save.setImage(R.drawable.tab_contacts_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Contacts").setIndicator(save), ContactsFragment.class, null);

    TabIndicator sponsoredContent = new TabIndicator(this);
    sponsoredContent.setImage(R.drawable.tab_keypad_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Keypad").setIndicator(sponsoredContent), KeypadFragment.class,
            null);
  }

  @Override
  public void onBackPressed() {
    int keypadPage = tabsAdapter.getCount() - 1;

    if (viewPager.getCurrentItem() != keypadPage) {
      viewPager.setCurrentItem(keypadPage);
    } else {
      isBackPressed = true;
      super.onBackPressed();
    }
  }

  @Override
  public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
      @NonNull int[] grantResults) {
    super.onRequestPermissionsResult(requestCode, permissions, grantResults);

    for (int i = 0; i < grantResults.length; i++) {
      if (grantResults[i] != PackageManager.PERMISSION_GRANTED) {
        return;
      }
    }

    setupViews();
  }

  private TabsAdapter.OnTabChangeListener onTabChangeListener = position -> {
    View view = MainActivity.this.getCurrentFocus();
    if (view != null) {
      InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
      imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
    }
  };

  @Override
  public void onStart() {
    super.onStart();
    EventBus.getDefault().register(this);
    VoIPClient.getInstance().addCallbacksListener(mCallbacksListener);
  }

  @Override
  public void onStop() {
    EventBus.getDefault().unregister(this);
    VoIPClient.getInstance().removeCallbacksListener(mCallbacksListener);
    super.onStop();
  }

  @Subscribe(sticky = true, threadMode = ThreadMode.MAIN)
  public void onMessageEvent(UpdateSettingsEvent event) {
    AppPrefs prefs = AppPrefs.getInstance();
    if (prefs == null) return;

    if (Utils.isRegistrationFree()) {
      subscribeRegistrationFree();
      VoIPClient.getInstance().unregister();
    } else {
      unSubscribeRegistrationFree();
      VoIPClient.getInstance().setConfigs();
      VoIPClient.getInstance().setStoredConfiguration();
    }

    EventBus.getDefault().removeStickyEvent(UpdateSettingsEvent.class);
  }

  // Only for registration free dialing feature
  private void subscribeRegistrationFree() {
    AppPrefs prefs = AppPrefs.getInstance();
    if (prefs == null) return;
    Log.d(TAG,"subscribing to reg-free");
    if (registrationFreeUser == null) {
      registrationFreeUser = FirebaseDatabase.getInstance().getReference()
          .child("subscribers")
          .child("incoming")
          .child(prefs.getString(AppPrefsKeys.USER_NAME));
      registrationFreeUser.addValueEventListener(databaseListener);
    }
  }

  // Only for registration free dialing feature
  private void unSubscribeRegistrationFree() {
    if (registrationFreeUser != null) {
      registrationFreeUser.removeEventListener(databaseListener);
      registrationFreeUser = null;
    }
  }

  private void clearFirebaseCall() {
    AppPrefs prefs = AppPrefs.getInstance();
    if (prefs == null) return;
    if (Utils.isRegistrationFree()) {
      DatabaseReference child = FirebaseDatabase.getInstance()
          .getReference()
          .child("subscribers")
          .child("incoming")
          .child(prefs.getString(AppPrefsKeys.USER_NAME));
      child.removeValue();
    }
  }

  private ValueEventListener databaseListener = new ValueEventListener() {

    @Override
    public void onDataChange(DataSnapshot dataSnapshot) {
      RegistrationFreeCall data = null;

      if (!dataSnapshot.hasChildren()) {
        return;
      }

      for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
        data = snapshot.getValue(RegistrationFreeCall.class);
      }

      if (data != null && !TextUtils.isEmpty(data.getCaller()) && !TextUtils.isEmpty(data.getSession())) {
        Log.d(TAG, "Notifying reg-free");
        VoIPClient.getInstance().notifyRegistrationFreeIncoming(data.getCaller(), data.getSession());
      }

      if (data == null) {
        Log.d(TAG, "data is null");
      }
      if (data != null && TextUtils.isEmpty(data.getCaller())) {
        Log.d(TAG, "No caller");
      }
      if (data != null && TextUtils.isEmpty(data.getSession())) {
        Log.d(TAG, "No session");
      }

      clearFirebaseCall();
    }

    @Override
    public void onCancelled(DatabaseError error) {
      Log.d(TAG, "Failed to read app title value.", error.toException());
    }

  };

  private IVoIPObserver mCallbacksListener = new IVoIPObserver() {

    @Override
    public void onSipStarted() {
      Log.d(TAG, "onSipStarted");

      if (Utils.isRegistrationFree()) {
        subscribeRegistrationFree();
      } else {
        unSubscribeRegistrationFree();
        VoIPClient.getInstance().registerAccount();
      }
    }

    @Override
    public void onSipStartFailed() {
      Log.d(TAG, "onSipStartFailed");
    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {
      switch (result) {
        case REGISTRATION_SUCCESS:
          SDKLogger.d(TAG, "Account is registered. Expire in " + expiry + " ms");
          break;
        case REGISTRATION_ERROR_CREDENTIALS:
          SDKLogger.d(TAG, "Account error unrecoverable");
          break;
        case REGISTRATION_UNREGISTERED:
          SDKLogger.d(TAG, "Account is unregistered");
          if (Utils.isRegistrationFree()) {
            SDKLogger.d(TAG, "Set configurations for registration-free calls");
            VoIPClient.getInstance().setConfigs();
            VoIPClient.getInstance().setStoredConfiguration();
          }
          break;
        default:
          Log.d(TAG, "onRegisterState");
          break;
      }
    }

    @Override
    public void onCallState(String callId, CallState callState, String contactNumber) {
      Log.d(TAG, "onCallState: [" + callId + "] " + callState);
      Intent callActivityIntent;
      switch (callState) {
        case CALL_STATE_RINGING:
          callActivityIntent = new Intent(MainActivity.this, OutgoingCallActivity.class);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_ID, callId);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_NUMBER, contactNumber);
          callActivityIntent.setFlags(Intent.FLAG_EXCLUDE_STOPPED_PACKAGES);
          if (mRingingCallId != null && mRingingCallId.equals(callId)) {
            return;
          } else if (mRingingCallId == null) {
            mRingingCallId = callId;
          }
          startActivity(callActivityIntent);
          break;
        case CALL_STATE_CONFIRMED:
        case CALL_STATE_EARLY:
          callActivityIntent = new Intent(MainActivity.this, CallActivity.class);
          callActivityIntent.putExtra("call_id", callId);
          callActivityIntent.putExtra("call_number", contactNumber);
          if (mRingingCallId != null && !mRingingCallId.equals(callId)) {
            return;
          }
          startActivity(callActivityIntent);
          break;
        case CALL_STATE_STARTING:
        case CALL_STATE_CONNECTING:
        case CALL_STATE_CALLING:
          callActivityIntent = new Intent(MainActivity.this, OutgoingCallActivity.class);
          mRingingCallId = callId;
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_ID, callId);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_NUMBER, contactNumber);
          startActivity(callActivityIntent);
          break;
        default:
          break;
      }
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {
      Log.d(TAG, "onSipStopped");
    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {
      Log.d(TAG, "onNetworkChanged");
    }

    @Override
    public void onNetworkRegained() {
      Log.d(TAG, "onNetworkRegained");
    }

    @Override
    public void onLicense(LicensingState state, String description) {
      SDKLogger.d(TAG, "License status - " + state + ". Cause - " + description);
      if (state == LicensingState.LICENSING_SUCCESS) {
        VoIPClient.getInstance().setConfigs();
        VoIPClient.getInstance().setStoredConfiguration();
      } else if (state == LicensingState.LICENSING_KEY_NETWORK_ERROR) {
        networkReceiver = new NetworkChangeReceiver();
        final IntentFilter filters = new IntentFilter();
        filters.addAction("android.net.conn.CONNECTIVITY_CHANGE");
        registerReceiver(networkReceiver, filters);
      }
    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }

  };

  public class NetworkChangeReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {
      if (isInitialStickyBroadcast()) {
        return;
      }
      new Handler().postDelayed(() -> {
        if (NetManager.isConnected(MainActivity.this)) {
          VoIPClient.initWithContext(getApplicationContext());
        }
      }, 500);
    }

  }

}
