package net.greenfieldtech.cloudonixsdk.cloudonixdialer.adapters;

import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TabHost;
import android.widget.TabWidget;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import java.util.ArrayList;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities.BaseActivity;

/**
 * TabsAdapter.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-16
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class TabsAdapter extends FragmentPagerAdapter implements TabHost.OnTabChangeListener,
    ViewPager.OnPageChangeListener {

  private final Context context;
  private final TabHost tabHost;
  private final ViewPager viewPager;
  private final ArrayList<TabInfo> tabInfoList = new ArrayList<>();
  private OnTabChangeListener onTabChangeListener;

  public TabsAdapter(final BaseActivity context, final TabHost tabHost, final ViewPager viewPager) {
    super(context.getSupportFragmentManager());

    this.context = context;
    this.tabHost = tabHost;
    this.viewPager = viewPager;
    this.tabHost.setOnTabChangedListener(this);

    viewPager.setAdapter(this);
    viewPager.addOnPageChangeListener(this);
    viewPager.setOffscreenPageLimit(4);
  }

  public void addTab(final TabHost.TabSpec tabSpec, final Class<?> clazz, final Bundle args) {
    tabSpec.setContent(new DummyTabFactory(context));
    String tag = tabSpec.getTag();

    TabInfo info = new TabInfo(tag, clazz, args);
    tabInfoList.add(info);
    tabHost.addTab(tabSpec);
    notifyDataSetChanged();
  }

  @Override
  public int getCount() {
    return tabInfoList.size();
  }

  @Override
  public Fragment getItem(final int position) {
    TabInfo info = tabInfoList.get(position);
    return Fragment.instantiate(context, info.clazz.getName(), info.args);
  }

  @Override
  public void onTabChanged(final String tabId) {
    int position = tabHost.getCurrentTab();
    viewPager.setCurrentItem(position);
    if (onTabChangeListener != null) {
      onTabChangeListener.onTabChanged(position);
    }
  }

  @Override
  public void onPageScrolled(final int position, final float positionOffset,
      final int positionOffsetPixels) {

  }

  @Override
  public void onPageSelected(final int position) {
    TabWidget widget = tabHost.getTabWidget();
    int oldFocusability = widget.getDescendantFocusability();
    widget.setDescendantFocusability(ViewGroup.FOCUS_BLOCK_DESCENDANTS);
    tabHost.setCurrentTab(position);
    widget.setDescendantFocusability(oldFocusability);
  }

  @Override
  public void onPageScrollStateChanged(final int state) {

  }

  public void setOnTabChangeListener(OnTabChangeListener onTabChangeListener) {
    this.onTabChangeListener = onTabChangeListener;
  }

  public interface OnTabChangeListener {

    void onTabChanged(int position);
  }

  static final class TabInfo {

    private final String tag;
    private final Class<?> clazz;
    private final Bundle args;

    public TabInfo(final String _tag, final Class<?> _class, final Bundle _args) {
      tag = _tag;
      clazz = _class;
      args = _args;
    }
  }

  static class DummyTabFactory implements TabHost.TabContentFactory {

    private final Context mContext;

    public DummyTabFactory(final Context context) {
      mContext = context;
    }

    @Override
    public View createTabContent(final String tag) {
      View v = new View(mContext);
      v.setMinimumWidth(0);
      v.setMinimumHeight(0);
      return v;
    }
  }

}
