package net.greenfieldtech.cloudonixsdk.cloudonixdialer.database;

import static nl.qbusict.cupboard.CupboardFactory.cupboard;

import android.content.ContentValues;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import java.util.Collections;
import java.util.List;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.Contact;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RecentCall;

/**
 * Created by alex on 21.04.16.
 */

/**
 * DatabaseHelper.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-04-21
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class DatabaseHelper extends SQLiteOpenHelper {

  private static final String DATABASE_NAME = "dialer.db";
  private static final int VERSION = 2;

  private static DatabaseHelper mInstance = null;

  static {
    cupboard().register(Contact.class);
    cupboard().register(RecentCall.class);
  }

  public static DatabaseHelper getInstance(Context context) {
    if (mInstance == null) {
      mInstance = new DatabaseHelper(context.getApplicationContext());
    }
    return mInstance;
  }

  public DatabaseHelper(Context context) {
    super(context, DATABASE_NAME, null, VERSION);
  }

  @Override
  public void onCreate(SQLiteDatabase db) {
    cupboard().withDatabase(db).createTables();
  }

  @Override
  public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
    cupboard().withDatabase(db).upgradeTables();
  }

  public void addContact(Contact contact) {
    cupboard().withDatabase(getWritableDatabase()).put(contact);
  }

  public void addCall(RecentCall call) {
    cupboard().withDatabase(getWritableDatabase()).put(call);
  }

  public void updateCall(RecentCall call) {
    ContentValues values = cupboard().withEntity(RecentCall.class).toContentValues(call);
    cupboard().withDatabase(getWritableDatabase()).update(RecentCall.class, values);
  }

  public void deleteContact(Contact contact) {
    cupboard().withDatabase(getWritableDatabase()).delete(contact);
  }

  public void deleteCall(RecentCall call) {
    cupboard().withDatabase(getWritableDatabase()).delete(call);
  }

  public List<Contact> getContacts() {
    List<Contact> contacts = cupboard().withDatabase(getReadableDatabase()).query(Contact.class)
        .list();
    return contacts;
  }

  public List<RecentCall> getAllCalls() {
    List<RecentCall> calls = cupboard().withDatabase(getReadableDatabase()).query(RecentCall.class)
        .list();
    Collections.reverse(calls);
    return calls;
  }

  public List<RecentCall> getMissedCalls() {
    List<RecentCall> calls = cupboard().withDatabase(getReadableDatabase()).query(RecentCall.class)
        .withSelection("isIncoming = 1 AND isMissed = 1")
        .list();
    Collections.reverse(calls);
    return calls;
  }

  public List<RecentCall> getNotViewedMissedCalls() {
    List<RecentCall> calls = cupboard().withDatabase(getReadableDatabase()).query(RecentCall.class)
        .withSelection("isIncoming = 1 AND isMissed = 1 AND isViewed = 0")
        .list();
    Collections.reverse(calls);
    return calls;
  }

}