package net.greenfieldtech.cloudonixsdk.cloudonixdialer.dialogs;

import android.app.Activity;
import android.app.Dialog;
import android.os.Bundle;
import androidx.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.DialogFragment;

import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;

/**
 * Created by alex on 23.05.16.
 */

/**
 * CallInCallDialog.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-23
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class CallInCallDialog extends DialogFragment {

  public static final String SECOND_NAME = "SECOND_NAME";
  public static final String SECOND_NUMBER = "SECOND_NUMBER";

  private View mDialogView;
  private String secondName;
  private String secondNumber;
  private SwitchDialogInterface mListener;

  public CallInCallDialog() {
  }

  public static CallInCallDialog newInstance(String secondName, String secondNumber) {
    CallInCallDialog fragment = new CallInCallDialog();
    Bundle args = new Bundle();
    args.putString(SECOND_NAME, secondName);
    args.putString(SECOND_NUMBER, secondNumber);
    fragment.setArguments(args);
    return fragment;
  }

  @Override
  public void onAttach(Activity activity) {
    super.onAttach(activity);
    try {
      mListener = (SwitchDialogInterface) activity;
    } catch (ClassCastException e) {
      throw new ClassCastException(activity.toString() + " must implement NoticeDialogListener");
    }
  }

  @Override
  public void onCreate(@Nullable Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    secondName = getArguments().getString(SECOND_NAME);
    secondNumber = getArguments().getString(SECOND_NUMBER);
  }

  @Override
  public Dialog onCreateDialog(Bundle savedInstanceState) {
    AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());

    LayoutInflater inflater = getActivity().getLayoutInflater();
    mDialogView = inflater.inflate(R.layout.dialog_call_in_call, null);

    ((TextView) mDialogView.findViewById(R.id.tv_name)).setText(secondName);
    ((TextView) mDialogView.findViewById(R.id.tv_number)).setText(secondNumber);

    mDialogView.findViewById(R.id.action_hangup_answer)
        .setOnClickListener(new View.OnClickListener() {
          @Override
          public void onClick(View v) {
            mListener.hangupAndAnswerSecondCall();
            dismiss();
          }
        });

    mDialogView.findViewById(R.id.action_hold_answer)
        .setOnClickListener(new View.OnClickListener() {
          @Override
          public void onClick(View v) {
            mListener.holdAndAnswerSecondCall();
            dismiss();
          }
        });

    mDialogView.findViewById(R.id.action_reject).setOnClickListener(new View.OnClickListener() {
      @Override
      public void onClick(View v) {
        mListener.rejectSecondCall();
        dismiss();
      }
    });

    builder.setView(mDialogView);

    return builder.create();
  }

}
