package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import androidx.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.google.android.material.snackbar.Snackbar;

import butterknife.BindView;
import butterknife.OnClick;
import butterknife.OnLongClick;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.Utils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;

/**
 * KeypadFragment.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-04-20
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class KeypadFragment extends BaseFragment {

  private static final String TAG = "PhoneDialer";
  private static final int NUMBER_LENGTH_LIMIT = 32;
  private static final int UPDATE_REGISTRATION_INTERVAL = 500;

  @BindView(R.id.root)
  RelativeLayout root;
  @BindView(R.id.tv_number)
  TextView tvNumber;
  @BindView(R.id.tv_registration_status)
  TextView tvRegStatus;

  private String registrationStatus;
  private String number;
  private boolean mIsFirstTime = true;
  private Handler updateRegistrationHandler;

  private float density;

  private String previousNumber;

  @Override
  public int getLayoutResource() {
    return R.layout.fragment_keypad;
  }

  @Override
  public void onCreate(@Nullable Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    density = getResources().getDisplayMetrics().density;

    number = "";
    updateRegistrationHandler = new Handler();
    updateRegistrationHandler.post(updateRegistrationRunnable);
  }

  @Override
  public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
    super.onViewCreated(view, savedInstanceState);

    VoIPClient.getInstance().setMediaResources(getActivity(), R.raw.busy, R.raw.ringing,
        R.raw.alert, R.raw.end_call, R.raw.hold, R.raw.call_on_hold, R.raw.call_off_hold);
  }

  @Override
  public void onResume() {
    super.onResume();

    tvNumber.setText(number);
    tvRegStatus.setText(registrationStatus);
    updateRegistrationHandler.postDelayed(updateRegistrationRunnable, UPDATE_REGISTRATION_INTERVAL);

    new Handler().postDelayed(() -> {
      if (getActivity() != null) {
        getActivity().runOnUiThread(mAskPermissionsRunnable);
      }
    }, 1000);
  }

  @Override
  public void onPause() {
    super.onPause();

    number = tvNumber.getText().toString();
    updateRegistrationHandler.removeCallbacks(updateRegistrationRunnable);
  }

  private Runnable updateRegistrationRunnable = new Runnable() {
    @Override
    public void run() {
      AppPrefs prefs = AppPrefs.getInstance();
      if (prefs == null) return;
      if (Utils.isRegistrationFree()) {
        String user = prefs.getString(AppPrefsKeys.USER_NAME);
        registrationStatus = String.format(getString(R.string.keypad_status_registration_free), user);
        tvRegStatus.setText(registrationStatus);
      } else {
        updateRegistrationStatus();
        updateRegistrationHandler.postDelayed(this, UPDATE_REGISTRATION_INTERVAL);
      }
    }
  };

  private Runnable mAskPermissionsRunnable = () -> {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
      if (mIsFirstTime) {
        mIsFirstTime = false;
        VoIPClient.getInstance().askForPermissions(getActivity(), true);
      } else {
        VoIPClient.getInstance().askForPermissions(getActivity(), mIsFirstTime);
      }
    }
  };

  private void updateRegistrationStatus() {
    AppPrefs prefs = AppPrefs.getInstance();
    if (prefs == null) return;
    if (VoIPClient.getInstance().isRegistered()) {
      String user = prefs.getString(AppPrefsKeys.USER_NAME);
      registrationStatus = String.format(getString(R.string.keypad_status_registered), user);
    } else {
      registrationStatus = getString(R.string.keypad_status_not_registered);
    }
    tvRegStatus.setText(registrationStatus);
  }

  @OnClick(R.id.btn_delete)
  void deleteClicked() {
    if (number.length() > 0) {
      number = number.substring(0, number.length() - 1);
      tvNumber.setText(number);
    }
    setDefaultTextSize();
  }

  @OnLongClick(R.id.btn_delete)
  boolean deleteNumber() {
    number = "";
    tvNumber.setText(number);
    setDefaultTextSize();
    return true;
  }

  private void setDefaultTextSize() {
    if (TextUtils.isEmpty(tvNumber.getText())) {
      tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP,
          getResources().getDimension(R.dimen.default_size) / density);
    }
  }

  @OnClick({R.id.btn_1, R.id.btn_2, R.id.btn_3, R.id.btn_4, R.id.btn_5, R.id.btn_6,
      R.id.btn_7, R.id.btn_8, R.id.btn_9, R.id.btn_0, R.id.btn_star, R.id.btn_pound})
  void onBtnNumberClick(View button) {
    if (tvNumber.getText().length() < NUMBER_LENGTH_LIMIT) {
      try {
        String viewTag = (String) button.getTag();
        try {
          int d = Integer.parseInt(viewTag);
          number = number + d;
          Log.i(TAG, "Number:" + number);
        } catch (NumberFormatException e) {
          if (viewTag.equals("*")) {
            number += "*";
          }
          if (viewTag.equals("#")) {
            number += "#";
          }
        }
        tvNumber.setText(number);
      } catch (ClassCastException e) {

      }
    } else {
      Snackbar.make(root, "Limit of number length", Snackbar.LENGTH_SHORT).show();
    }
  }

  @OnLongClick(R.id.btn_0)
  boolean addPlusSymbol() {
    number = number + "+";
    tvNumber.setText(number);
    return true;
  }

  @OnClick(R.id.btn_call)
  void doCall() {
    AppPrefs prefs = AppPrefs.getInstance();
    if (prefs == null) return;

    if (!SdkFacade.isCodecSelected()) {
      Snackbar.make(root, getString(R.string.error_no_codec), Snackbar.LENGTH_SHORT).show();
      return;
    }

    if (TextUtils.isEmpty(number)) {
      // set previous number
      if (previousNumber != null) {
        number = previousNumber;
        tvNumber.setText(previousNumber);
      }
    } else {
      if (Utils.isRegistrationFree()) {
        VoIPClient.getInstance().dialRegistrationFree(number);
      } else {
        if (VoIPClient.getInstance().isRegistered()) {
          previousNumber = number;
          VoIPClient.getInstance().dial(number);
        } else {
          Snackbar
              .make(root, getString(R.string.error_no_registered), Snackbar.LENGTH_SHORT)
              .show();
        }
      }
    }
  }

}
