package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.preference.ListPreference;
import androidx.preference.Preference;
import androidx.preference.Preference.OnPreferenceChangeListener;
import androidx.preference.PreferenceCategory;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.PreferenceGroupAdapter;
import androidx.preference.PreferenceScreen;
import androidx.preference.PreferenceViewHolder;
import androidx.preference.SwitchPreference;
import androidx.recyclerview.widget.RecyclerView.Adapter;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;

/**
 * SettingsFragment.java
 * 
 * Authors:
 *   - Aleksei Dzis, 2016-08-03
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class SettingsFragment extends PreferenceFragmentCompat implements
    OnPreferenceChangeListener {

  private AppPrefs prefs = AppPrefs.getInstance();

  @Override
  protected Adapter onCreateAdapter(PreferenceScreen preferenceScreen) {
    return new PreferenceGroupAdapter(preferenceScreen) {
      @Override
      public void onBindViewHolder(@NonNull PreferenceViewHolder holder, int position) {
        super.onBindViewHolder(holder, position);
        Preference preference = getItem(position);
        if (preference instanceof PreferenceCategory) {
          setZeroPaddingToLayoutChildren(holder.itemView);
        } else {
          View iconFrame = holder.itemView.findViewById(R.id.icon_frame);
          if (iconFrame != null) {
            iconFrame.setVisibility(preference.getIcon() == null ? View.GONE : View.VISIBLE);
          }
        }
      }
    };
  }

  private void setZeroPaddingToLayoutChildren(View view) {
    if (!(view instanceof ViewGroup)) {
      return;
    }

    ViewGroup viewGroup = (ViewGroup) view;
    int childCount = viewGroup.getChildCount();
    for (int i = 0; i < childCount; i++) {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
        viewGroup.setPaddingRelative(0, viewGroup.getPaddingTop(), viewGroup.getPaddingEnd(), viewGroup.getPaddingBottom());
      } else {
        viewGroup.setPadding(0, viewGroup.getPaddingTop(), viewGroup.getPaddingRight(), viewGroup.getPaddingBottom());
      }
    }
  }

  @Override
  public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
    addPreferencesFromResource(R.xml.preferences_screen);

    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.SERVER));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USER_NAME));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.DOMAIN));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.DISPLAY_NAME));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.PASSWORD));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.TRANSPORT));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.TRANSPORT_PORT));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_DNS_SRV));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_ICE));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_REGISTRATION_FREE));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.MIN_RATE));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.MAX_RATE));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_OPUS));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_ILBC));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_G722));
    bindPreferenceSummaryToValue(findPreference(AppPrefsKeys.USE_G711));
  }

  private void bindPreferenceSummaryToValue(@Nullable Preference preference) {
    if (preference == null) {
      return;
    }
    preference.setOnPreferenceChangeListener(this);
    onPreferenceChange(preference, (preference instanceof SwitchPreference) ?
        prefs.getBoolean(preference.getKey()) : prefs.getString(preference.getKey()));
  }

  @Override
  public boolean onPreferenceChange(Preference preference, Object value) {
    String stringValue = value.toString();
    String key = preference.getKey();

    switch (key) {
      case AppPrefsKeys.USE_ICE:
      case AppPrefsKeys.USE_OPUS:
      case AppPrefsKeys.USE_ILBC:
      case AppPrefsKeys.USE_G722:
      case AppPrefsKeys.USE_G711:
        break;
    }

    if (preference instanceof ListPreference) {
      ListPreference listPreference = (ListPreference) preference;
      int index = listPreference.findIndexOfValue(stringValue);
      preference.setSummary(index >= 0 ? listPreference.getEntries()[index] : null);
    } else {
      preference.setSummary(stringValue);
    }
    return true;
  }

}
