package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.content.Intent;
import android.os.Bundle;
import androidx.annotation.Nullable;
import android.view.View;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import butterknife.BindView;
import butterknife.OnClick;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities.SettingsActivity;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;

/**
 * StatusFragment.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-04-20
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class StatusFragment extends BaseFragment {

  public static final int REQUEST_EDIT_PREFS = 1001;

  @BindView(R.id.root)
  RelativeLayout root;
  @BindView(R.id.tv_server)
  TextView tvServer;
  @BindView(R.id.tv_port)
  TextView tvPort;
  @BindView(R.id.tv_protocol)
  TextView tvProtocol;
  @BindView(R.id.tv_username)
  TextView tvUsername;
  @BindView(R.id.tv_domain)
  TextView tvDomain;
  @BindView(R.id.tv_password)
  TextView tvPassword;
  @BindView(R.id.tv_display_name)
  TextView tvDisplayName;
  @BindView(R.id.tv_registration)
  TextView tvRegistration;
  @BindView(R.id.tv_ice)
  TextView tvIce;
  @BindView(R.id.tv_dns_srv)
  TextView tvDns;
  @BindView(R.id.tv_codec)
  TextView tvCodec;
  @BindView(R.id.tv_user_agent)
  TextView tvUseragent;
  @BindView(R.id.tv_maximal_rate)
  TextView tvMaxRate;
  @BindView(R.id.tv_minimal_rate)
  TextView tvMinRate;

  @BindView(R.id.iv_registration)
  ImageView ivRegistration;
  @BindView(R.id.iv_ice)
  ImageView ivIce;
  @BindView(R.id.iv_dns_srv)
  ImageView iv_Dns;

  @Override
  public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
    super.onViewCreated(view, savedInstanceState);
    setInfo();
  }

  @Override
  public int getLayoutResource() {
    return R.layout.fragment_status;
  }

  @OnClick(R.id.btn_edit)
  void edit() {
    startActivityForResult(new Intent(getActivity(), SettingsActivity.class), REQUEST_EDIT_PREFS);
  }

  @Override
  public void onActivityResult(int requestCode, int resultCode, Intent data) {
    super.onActivityResult(requestCode, resultCode, data);
    if (requestCode == REQUEST_EDIT_PREFS) {
      setInfo();
    }
  }

  private void setInfo() {
    AppPrefs prefs = AppPrefs.getInstance();

    tvServer.setText(prefs.getString(AppPrefsKeys.SERVER));
    tvPort.setText(prefs.getString(AppPrefsKeys.TRANSPORT_PORT));
    tvProtocol.setText(prefs.getString(AppPrefsKeys.TRANSPORT));
    tvUsername.setText(prefs.getString(AppPrefsKeys.USER_NAME));
    tvPassword.setText(prefs.getString(AppPrefsKeys.PASSWORD));
    tvDisplayName.setText(prefs.getString(AppPrefsKeys.DISPLAY_NAME));
    tvDomain.setText(prefs.getString(AppPrefsKeys.DOMAIN));
    tvIce.setText(prefs.getBoolean(AppPrefsKeys.USE_ICE) ? "YES" : "NO");
    ivIce.setImageResource(
        prefs.getBoolean(AppPrefsKeys.USE_ICE) ?
            R.drawable.ic_settings_check :
            R.drawable.ic_settings_cross);
    tvDns.setText(prefs.getBoolean(AppPrefsKeys.USE_DNS_SRV) ? "YES" : "NO");
    iv_Dns.setImageResource(
        prefs.getBoolean(AppPrefsKeys.USE_DNS_SRV) ?
            R.drawable.ic_settings_check :
            R.drawable.ic_settings_cross);
    tvCodec.setText(SdkFacade.getEnabledCodecs());
    tvUseragent.setText("Greenfield");
    tvMinRate.setText(prefs.getString(AppPrefsKeys.MIN_RATE));
    tvMaxRate.setText(prefs.getString(AppPrefsKeys.MAX_RATE));
  }

}
