package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.text.TextUtils;
import java.util.ArrayList;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.appinterface.VoipCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefs;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs.AppPrefsKeys;

/**
 * SdkFacade.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-04
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class SdkFacade {

  public static boolean isCodecSelected() {
    AppPrefs prefs = AppPrefs.getInstance();
    return prefs.getBoolean(AppPrefsKeys.USE_OPUS) ||
           prefs.getBoolean(AppPrefsKeys.USE_ILBC) ||
           prefs.getBoolean(AppPrefsKeys.USE_G722) ||
           prefs.getBoolean(AppPrefsKeys.USE_G711);
  }

  public static String getEnabledCodecs() {
    AppPrefs prefs = AppPrefs.getInstance();
    String opus = prefs.getBoolean(AppPrefsKeys.USE_OPUS) ? "Opus" : "";
    String iLBC = prefs.getBoolean(AppPrefsKeys.USE_ILBC) ? "iLBC" : "";
    String g722 = prefs.getBoolean(AppPrefsKeys.USE_G722) ? "G722" : "";
    String g711 = prefs.getBoolean(AppPrefsKeys.USE_G711) ? "G711" : "";

    ArrayList<String> codecArray = new ArrayList<>();
    if (!TextUtils.isEmpty(opus)) {
      codecArray.add(opus);
    }
    if (!TextUtils.isEmpty(iLBC)) {
      codecArray.add(iLBC);
    }
    if (!TextUtils.isEmpty(g722)) {
      codecArray.add(g722);
    }
    if (!TextUtils.isEmpty(g711)) {
      codecArray.add(g711);
    }
    return TextUtil.getCodecString(codecArray);
  }

  public static String findActiveCall(String currentCallId) {
    String result = null;
    VoIPClient client = VoIPClient.getInstance();
    if (client != null && client.isRegistered()) {
      VoipCall[] calls = client.getCalls();
      for (int i = 0; i < calls.length; i++) {
        if (calls[i].callState == IVoIPObserver.CallState.CALL_STATE_MEDIAACTIVE) {
          result = calls[i].key;
        }
      }
    }
    if (result == null) {
      return currentCallId;
    }
    return result;
  }

}
