package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.content.Context;
import android.graphics.Typeface;
import android.util.SparseArray;

/**
 * .java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-18
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class TypefaceManager {

  public final static int OPEN_SANS_BOLD = 0;
  public final static int OPEN_SANS_BOLD_ITALIC = 1;
  public final static int OPEN_SANS_EXTRA_BOLD = 2;
  public final static int OPEN_SANS_EXTRA_BOLD_ITALIC = 3;
  public final static int OPEN_SANS_ITALIC = 4;
  public final static int OPEN_SANS_LIGHT = 5;
  public final static int OPEN_SANS_LIGHT_ITALIC = 6;
  public final static int OPEN_SANS_REGULAR = 7;
  public final static int OPEN_SANS_SEMI_BOLD = 8;
  public final static int OPEN_SANS_SEMI_BOLD_ITALIC = 9;

  /**
   * Array of created typefaces for later reused.
   */
  private final static SparseArray<Typeface> mTypefaces = new SparseArray<>(20);

  /**
   * Obtain typeface.
   *
   * @param context The Context the widget is running in, through which it can access the current
   * theme, resources, etc.
   * @param typefaceValue The value of "typeface" attribute
   * @return specify {@link android.graphics.Typeface}
   * @throws IllegalArgumentException if unknown `typeface` attribute value.
   */
  public static Typeface obtainTypeface(Context context, int typefaceValue)
      throws IllegalArgumentException {
    Typeface typeface = mTypefaces.get(typefaceValue);
    if (typeface == null) {
      typeface = createTypeface(context, typefaceValue);
      mTypefaces.put(typefaceValue, typeface);
    }
    return typeface;
  }

  /**
   * Create typeface from assets.
   *
   * @param context The Context the widget is running in, through which it can access the current
   * theme, resources, etc.
   * @param typefaceValue The value of "typeface" attribute
   * @return typeface {@link android.graphics.Typeface}
   * @throws IllegalArgumentException if unknown `typeface` attribute value.
   */
  private static Typeface createTypeface(Context context, int typefaceValue)
      throws IllegalArgumentException {
    Typeface typeface;
    switch (typefaceValue) {
      case OPEN_SANS_BOLD:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-Bold.ttf");
        break;
      case OPEN_SANS_BOLD_ITALIC:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-BoldItalic.ttf");
        break;
      case OPEN_SANS_EXTRA_BOLD:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-ExtraBold.ttf");
        break;
      case OPEN_SANS_EXTRA_BOLD_ITALIC:
        typeface = Typeface
            .createFromAsset(context.getAssets(), "fonts/OpenSans-ExtraBoldItalic.ttf");
        break;
      case OPEN_SANS_ITALIC:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-Italic.ttf");
        break;
      case OPEN_SANS_LIGHT:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-Light.ttf");
        break;
      case OPEN_SANS_LIGHT_ITALIC:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-LightItalic.ttf");
        break;
      case OPEN_SANS_REGULAR:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-Regular.ttf");
        break;
      case OPEN_SANS_SEMI_BOLD:
        typeface = Typeface.createFromAsset(context.getAssets(), "fonts/OpenSans-Semibold.ttf");
        break;
      case OPEN_SANS_SEMI_BOLD_ITALIC:
        typeface = Typeface
            .createFromAsset(context.getAssets(), "fonts/OpenSans-SemiboldItalic.ttf");
        break;
      default:
        throw new IllegalArgumentException("Unknown `typeface` attribute value " + typefaceValue);
    }
    return typeface;
  }


}