package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.content.Context;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.TextPaint;
import android.text.style.AbsoluteSizeSpan;
import android.text.style.ForegroundColorSpan;
import android.text.style.MetricAffectingSpan;
import android.util.TypedValue;

/**
 * UIUtils.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-20
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class UIUtils {

  public static Spannable getTypefaceSpannable(Context context, String title, int textSize,
      Typeface typeface) {
    float size = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, textSize,
        context.getResources().getDisplayMetrics());
    SpannableString sb = new SpannableString(title);
    int start = 0;
    int end = sb.length();
    sb.setSpan(new CustomTypefaceSpan(typeface), start, end, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
    sb.setSpan(new AbsoluteSizeSpan((int) size), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
    return sb;
  }

  public static Spannable getTypefaceSpannable(Context context, String title, int textSize,
      int start, int end, int color, Typeface typeface) {
    float size = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, textSize,
        context.getResources().getDisplayMetrics());

    SpannableString sb = new SpannableString(title);
    sb.setSpan(new ForegroundColorSpan(context.getResources().getColor(color)), start, end,
        Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
    sb.setSpan(new CustomTypefaceSpan(typeface), start, end, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
    sb.setSpan(new AbsoluteSizeSpan((int) size), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
    return sb;
  }

  private static class CustomTypefaceSpan extends MetricAffectingSpan {

    private final Typeface typeface;

    public CustomTypefaceSpan(final Typeface typeface) {
      this.typeface = typeface;
    }

    @Override
    public void updateDrawState(final TextPaint drawState) {
      apply(drawState);
    }

    @Override
    public void updateMeasureState(final TextPaint paint) {
      apply(paint);
    }

    private void apply(final Paint paint) {
      final Typeface oldTypeface = paint.getTypeface();
      final int oldStyle = oldTypeface != null ? oldTypeface.getStyle() : 0;
      final int fakeStyle = oldStyle & ~typeface.getStyle();

      if ((fakeStyle & Typeface.BOLD) != 0) {
        paint.setFakeBoldText(true);
      }

      if ((fakeStyle & Typeface.ITALIC) != 0) {
        paint.setTextSkewX(-0.25f);
      }

      paint.setTypeface(typeface);
    }
  }

}
