package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.prefs;

import android.content.Context;
import android.content.SharedPreferences;
import androidx.preference.PreferenceManager;

/**
 * AppPrefs.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2019-09-26
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class AppPrefs implements PrefsInterface {

  private static AppPrefs instance;
  private final SharedPreferences prefs;

  public static void init(Context context) {
    if (instance == null) {
      instance = new AppPrefs(context);
    }
  }

  private AppPrefs(Context context) {
    prefs = PreferenceManager.getDefaultSharedPreferences(context);
  }

  public static AppPrefs getInstance() {
    return instance;
  }

  @Override
  public void putString(String key, String value) {
    prefs.edit().putString(key, value).apply();
  }

  @Override
  public String getString(String key) {
    return prefs.getString(key, DEFAULT_STRING_VALUE);
  }

  @Override
  public String getString(String key, String defaultValue) {
    return prefs.getString(key, defaultValue);
  }

  @Override
  public void putInt(String key, int value) {
    prefs.edit().putInt(key, value).apply();
  }

  @Override
  public int getInt(String key) {
    return prefs.getInt(key, DEFAULT_INT_VALUE);
  }

  @Override
  public int getInt(String key, int defaultValue) {
    return prefs.getInt(key, defaultValue);
  }

  @Override
  public void putLong(String key, long value) {
    prefs.edit().putLong(key, value).apply();
  }

  @Override
  public long getLong(String key) {
    return prefs.getLong(key, DEFAULT_INT_VALUE);
  }

  @Override
  public long getLong(String key, int defaultValue) {
    return prefs.getLong(key, defaultValue);
  }

  @Override
  public void putBoolean(String key, boolean value) {
    prefs.edit().putBoolean(key, value).apply();
  }

  @Override
  public boolean getBoolean(String key) {
    return prefs.getBoolean(key, DEFAULT_BOOLEAN_VALUE);
  }

  @Override
  public boolean getBoolean(String key, boolean defaultValue) {
    return prefs.getBoolean(key, defaultValue);
  }

  @Override
  public void clearCurrentPrefs(String key) {
    prefs.edit().remove(key).apply();
  }

  @Override
  public void clear() {
    prefs.edit().clear().apply();
  }

}
