package net.greenfieldtech.cloudonixsdk.cloudonixdialer.views;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Typeface;
import android.util.AttributeSet;

import androidx.appcompat.widget.AppCompatEditText;

import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.TypefaceManager;

/**
 * TypefaceEditText.java
 * 
 * Authors:
 *   - Alexander Rubanskiy, 2016-05-20
 * 
 * Copyright © 2015 GreenfieldTech 2015 Ltd. All rights reserved.
 * Copyright © 2018 Cloudonix Ltd. All rights reserved.
 * Copyright © 2019 Cloudonix Inc. All rights reserved.
 */
public class TypefaceEditText extends AppCompatEditText {

  public TypefaceEditText(Context context) {
    super(context);
    onInitTypeface(context, null, 0);
  }

  public TypefaceEditText(Context context, AttributeSet attrs) {
    super(context, attrs);
    onInitTypeface(context, attrs, 0);
  }

  public TypefaceEditText(Context context, AttributeSet attrs, int defStyleAttr) {
    super(context, attrs, defStyleAttr);
    onInitTypeface(context, attrs, defStyleAttr);
  }

  private void onInitTypeface(Context context, AttributeSet attrs, int defStyle) {
    // Typeface.createFromAsset doesn't work in the layout editor, so skipping.
    if (isInEditMode()) {
      return;
    }

    int typefaceValue = TypefaceManager.OPEN_SANS_ITALIC;
    if (attrs != null) {
      TypedArray values = context
          .obtainStyledAttributes(attrs, R.styleable.TypefaceTextView, defStyle, 0);
      if (values != null) {
        typefaceValue = values.getInt(R.styleable.TypefaceTextView_typeface, typefaceValue);
        values.recycle();
      }
    }

    Typeface typeface = TypefaceManager.obtainTypeface(context, typefaceValue);
    setTypeface(typeface);
  }

}
